/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EquinoxFramework.java 3054 2008-04-30 15:41:13Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.osgi.plugins.launcher.equinox;

import java.io.File;
import java.net.URL;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.adaptor.EclipseStarter;
import org.osgi.framework.BundleContext;
import org.ow2.easybeans.osgi.plugins.launcher.api.FrameworkException;
import org.ow2.easybeans.osgi.plugins.launcher.api.IFramework;
import org.ow2.util.file.FileUtils;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Implementation of the launcher for Eclipse Equinox.
 * @author Florent BENOIT
 */
public class EquinoxFramework implements IFramework {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EquinoxFramework.class);


    /**
     * Working directory.
     */
    private File workingDirectory = null;

    /**
     * Configuration file path.
     */
    private URL config = null;

    /**
     * Arguments to send to the framework.
     */
    private String[] args = null;

    /**
     * Configure the Equinox gateway by using the given configuration file.
     * @param config the URL of the configuration file to use
     * @throws FrameworkException if the configuration is not done
     */
    public void configure(final URL config) throws FrameworkException {
       // Keep URL config file
        this.config = config;
    }

    /**
     * Creates the arguments required to launch the framework.
     */
    protected void createArgs() {
        List<String> arguments = new LinkedList<String>();

        // Add Configuration
        arguments.add("-configuration");
        arguments.add(new File(config.getFile()).getParent());

        // For data
        arguments.add("-data");
        arguments.add(workingDirectory.getAbsolutePath());

        // Clean data
        arguments.add("-clean");

        this.args = arguments.toArray(new String[arguments.size()]);
    }

    /**
     * Start the framework.
     * @throws FrameworkException if the framework can't be started.
     */
    public void start() throws FrameworkException {

        // Cleanup
        if (workingDirectory.exists()) {
            FileUtils.delete(workingDirectory);
        }

        // Create arguments
        createArgs();

        logger.info("Using arguments {0}", Arrays.asList(args));

        // Start it
        try {
            EclipseStarter.startup(args, null);
        } catch (Exception e) {
            throw new FrameworkException("Cannot start Eclipse Equinox", e);
        }
    }


    /**
     * Stop the framework.
     * @throws FrameworkException if the framework can't be stopped.
     */
    public void stop() throws FrameworkException {
        // Stop Equinox instance
        try {
            EclipseStarter.shutdown();
        } catch (Exception e) {
            throw new FrameworkException("Cannot stop Eclipse Equinox", e);
        }
    }

    /**
     * Gets the bundle context of the framework.
     * @return the bundle context allowing to do some operations.
     */
    public BundleContext getBundleContext() {
        return EclipseStarter.getSystemBundleContext();
    }

    /**
     * Checks if the framework is running.
     * @return true if the framework is running
     */
    public boolean isRunning() {
        return EclipseStarter.isRunning();
    }


    /**
     * Sets the working directory.
     * @param workingDirectory the given directory used to store some files.
     */
    public void setWorkingDirectory(final File workingDirectory) {
        this.workingDirectory = workingDirectory;
    }

}
